class GestureInventory extends Inventory;

//class for clientside only gesturing functions

#exec OBJ LOAD FILE=..\Textures\EmitterTextures.utx

//***variables for spawning effect when hand draws on screen
var vector spawnTemp, spawnTo, playerCamLoc; 
	//temp vector for ScreenToWorld, spawnTo for final location
var xEmitter gEffect; //gesture effect actor
var xEmitter hEffect; //hold effect actor
var grabinPawn P; //the pawn this actor belongs to
var Interaction gInteraction;	//interaction for gesturing
var bool cClick, cDraw, doDrawSnd; //if Rmouse is held, if Lmouse is held, if draw sound plays
var bool cRelease; //this tells a held attack to be executed [in PRI]
var bool assignInt; //true if interaction hasnt been assigned yet
var gestureInter ir;
var grabinController PC; //these 2 to make manipulation more painless

var vector prevVec; //this for the old mouse coords
var bool firstTime; //first time a gesture is done? if so, init prevVec
var int ticks; //how many ticks has mouse not moved for?
var float tolerance; 
	//gesturing will only be calculated when mouse pointer is moved by >tolerance
var int mouseSitTolerance;
	//how many ticks the mouse is stationary before prevVec is reset
var() float holdTime; //number of seconds held gestures stay active for
var int countDown; //countdown int to be passed to HUD
var array<int> gestureID; //the array to be passed to PRI to see what happens

replication
{
	reliable if(ROLE < Role_Authority)
		countDown;
}

//===================================
//======AUXILIARY FUNCTIONS==========
//===================================

replication
{
}

simulated function destroyed()
{
	//kill the gesture buffer before recycle
	if (gestureID.Length != 0)
	{
		gestureID.Remove(0, gestureID.Length);
		gestureID.Length = 0;
	}
	Super.destroyed();
}

simulated function Tick(float deltatime)
{	
	if (gInteraction == none)
		assignInt = true;
	if (assignInt == true)
	{
		getInteraction();
		ir = gestureInter(gInteraction);
		assignInt = false;
	} else {
	if ( PC != none && P != none)	//no bot support for gestures yet, also prevents spectator tick
	{
		//log("TICKTICKTICK :: :: " $ ir);
		if(cClick)
   		{
			grabinPRI(PC.PlayerReplicationInfo).playAmbient(true);
			//play sndFX
			cRelease = false;
			if (isinState('holdProjAttack'))
				gotoState('');
			drawing();
			if (!cDraw)
			{
				grabinPRI(PC.PlayerReplicationInfo).drawCursor(gEffect, spawnTo, playerCamLoc); 
					//only draw cursor when not actually drawing shapes
				gotoState(''); //this stops calculation
			}
		} else {
			grabinPRI(PC.PlayerReplicationInfo).playAmbient(false);
			if (isinState('gesturing'))
				gotoState(''); //this stops calculation
			
			if (gestureID.Length != 0)
			{  //send gestureID, then purge out the old gesture buffer
				grabinPRI(PC.PlayerReplicationInfo).interp(gestureID);
				gestureID.Remove(0, gestureID.Length);
				gestureID.Length = 0;
			}
		}
   		if (cDraw)
  		{
   			if (!isinState('gesturing'))
   				gotoState('gesturing');
   				//handle gesture calculation and effect
   		}
   		if (cRelease)
   			if (isinState('holdProjAttack'))
   				gotoState('');
   				
		if ( doDrawSnd == true ) //set by interaction
		{
			grabinPRI(PC.PlayerReplicationInfo).playRelease();
			doDrawSnd = false;
				//reset so it doesnt loop sounds
		}
	}
	}
}

simulated function setupGesture()
{	
	setBase(P);	//set the inv to follow pawn
	PC = grabinController(P.Controller);
	//log("PC IS! " $ PC);
	//log("PAWN IS! " $ P);
}

simulated function getInteraction()
{
	local PlayerController PCn;
	local int i; //iterator
	
	foreach DynamicActors(class'PlayerController', PCn)
	{
		if ( Viewport(PCn.Player) != None )
        	{
        		for (i = 0; i < PCn.Player.LocalInteractions.Length; i++)
            		{
            			if ( PCn.Player.LocalInteractions[i].IsA('gestureInter') )
            				gInteraction = PCn.Player.LocalInteractions[i];
            		}
            	}
        }
}

//===================================
//======EFFECT RENDERING FUNCTIONS===
//===================================

simulated function drawing()
{
	playerCamLoc = P.Location + P.EyePosition();
   		//set location of player cam before function execution
	spawnTemp.X = PC.Player.WindowsMouseX;
   	spawnTemp.Y = PC.Player.WindowsMouseY;
   		//vector to spawn effect to in relation to camera
   	spawnTo = ir.ScreenToWorld(spawnTemp,playerCamLoc,);// + (5 * vector(P.GetViewRotation()));
}

//===================================
//====-==CALCULATION FUNCTIONS=======
//===================================

state gesturing

	//====================================================================
	// this state is responsible for calculating the gesture ID array to 
	// be passed to the PRI to see what happens. the array generated
	// takes this form:
	//	INDEX		NUMBER
	//	0		0	(zero indicates first pen down)
	//	1		First directional number
	//	...		Second one... etc
	//	...		0	This means the 'pen' was lifted and placed again
	//	...		More numbers for the next shape drawn
	//	Length-1	Last number in last gesture
	//====================================================================
{	
   	simulated function BeginState()
   	{
		//log("   A NEW SHAPE A-HAPPENS ");
		firstTime = true;
		//get tolerance of mouse movement
		tolerance = calcTolerance();
		setTimer(0.02,true);
   	}

   	simulated function Timer()
   	{
	     	//vector calculation variables
		local vector newVec, diffVec;
		local rotator angleBetween;
		local float diffSize;
		local int newDir; //computed direction ID
	     
	     grabinPRI(PC.PlayerReplicationInfo).drawEffect(hEffect, spawnTo, playerCamLoc);
   				//draw effect onscreen
   		
   		if (firstTime)
   		{
   			  //return values to defaults
   			prevVec.X = PC.Player.WindowsMouseX;
   			prevVec.Y = PC.Player.WindowsMouseY;
   			prevVec.Z = 0;
   			diffVec = vect(0,0,0);
   			
   			  //add a blank to the end of the array for pen down
   			gestureID.insert(gestureID.Length, 1);
			gestureID[gestureID.Length - 1] = 0; 
			ticks = 0;
			
   			firstTime = false;
   		}
   		newVec.X = PC.Player.WindowsMouseX;
   		newVec.Y = PC.Player.WindowsMouseY;
   		newVec.Z = 0;
   			//set up the vector holding mouse location
   		diffSize = VSize(newVec - prevVec);   		
   		if (diffSize > tolerance)
		{
   		  //if distance is above tolerance change diff vector and reset prevVec
   			
   			//log("   ticks are am at " $ ticks);
   			
   			ticks = 0;  //reset ticks of non-mouse movement
   			diffVec = newVec - prevVec;
			prevVec = newVec;
		  //find angle between 2 vectors and compute
   			angleBetween = rotator(normal(diffVec));   			
   			newDir = computeOutput(angleBetween);
   			if ( gestureID[gestureID.Length-1] != newDir )
			{  //if direction has changed
			    gestureID.insert(gestureID.Length, 1);	//add an element
			    gestureID[gestureID.Length - 1] = newDir; //fill the new element
			}
   		}
   		else if (ticks > mouseSitTolerance)
   		{
   			prevVec.X = PC.Player.WindowsMouseX;
   			prevVec.Y = PC.Player.WindowsMouseY;
   			prevVec.Z = 0;
   			ticks = 0;
   			 //this stops us getting rounded corners on straight shapes, though
   			 // a small pause in gesturing is now necessary on sharp corners.
   		}
   		ticks++;
	}

     	simulated function EndState()
     	{
     	}
	
	function int computeOutput(rotator angle)
	{
		// rotator notes:
		// 8192 == 45 degrees
		// TOLERANCES:
		// 1	61441 to 65535 
		//	 && 0 to 4095
		// 2	4096 to 12288
		// 3	12289 to 20480		
		// 4	20481 to 28672		   6 7 8
		// 5	28673 to 32768		    \|/
		//	 && -32678 to -28673	 5 --+-- 1
		// 6	-20481 to -28672	    /|\
		// 7	-12289 to -20480	   4 3 2
		// 8	-4096 to -12288
		local int Y; //angle's Y val, makes things easier
		
		Y = angle.Yaw;
		
		if ( 12288 >= Y && Y >= 4096 )
			return 2;
		else if ( 20480 >= Y && Y >= 12289 )
			return 3;
		else if ( 28672 >= Y && Y >= 20481 )
			return 4;
		else if ( (32768 >= Y && Y >= 28673) || (-32768 <= Y && Y <= -28673) )
			return 5;
		else if ( -28672 <= Y && Y <= -20481 )
			return 6;
		else if ( -20480 <= Y && Y <= -12289 )
			return 7;
		else if ( -12288 <= Y && Y <= -4096 )
			return 8;
		else
			return 1;		//find out which direction we draw in
	}
	
	function float calcTolerance()
	{
		local string CurrentRes;
			//the screen resolution check for per-pixel tolerance calc
		CurrentRes = PC.ConsoleCommand("GETCURRENTRES");
		switch (CurrentRes)
		{
			case "320x240":
				tolerance = 24;
				break;
			case "512x384":
				tolerance = 38;
				break;
			case "640x480":
				tolerance = 48;
				break;
			//larger resolutions can be the same tolerance
			default:
				tolerance = 50;
				break;
		}
		return tolerance;
	}
}

state holdProjAttack
{  //hold the projectile attack until the user clicks again or 5 seconds
	simulated function beginState()
	{
		countDown = holdTime;
		setTimer(1.0,true);
	}
	
	simulated function timer()
	{
		countDown--;
		if (countDown <= 0)
			gotoState('');
	}
	
	simulated function endState()
	{
		if ( cRelease )
			grabinPRI(PC.PlayerReplicationInfo).DoProjectile();
		cRelease = false;
		countDown = holdTime+1;
	}
}

//===================================
//============DEFAULTS===============
//===================================

defaultproperties
{
	mouseSitTolerance=8
	holdTime=5.0
}
